//
//  SKPSMTPMessage.h
//
//  Created by Ian Baird on 10/28/08.
//
//  Copyright (c) 2008 Skorpiostech, Inc. All rights reserved.
//
//  Permission is hereby granted, free of charge, to any person
//  obtaining a copy of this software and associated documentation
//  files (the "Software"), to deal in the Software without
//  restriction, including without limitation the rights to use,
//  copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the
//  Software is furnished to do so, subject to the following
//  conditions:
//
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
//

#import <Foundation/Foundation.h>
#import <CoreServices/CoreServices.h>

enum 
{
    kSKPSMTPIdle = 0,
    kSKPSMTPConnecting,
    kSKPSMTPWaitingEHLOReply,
    kSKPSMTPWaitingTLSReply,
    kSKPSMTPWaitingLOGINUsernameReply,
    kSKPSMTPWaitingLOGINPasswordReply,
	kSKPSMTPWaitingCRAMMD5Reply,
    kSKPSMTPWaitingAuthSuccess,
    kSKPSMTPWaitingFromReply,
    kSKPSMTPWaitingToReply,
    kSKPSMTPWaitingForEnterMail,
    kSKPSMTPWaitingSendSuccess,
    kSKPSMTPWaitingQuitReply,
    kSKPSMTPMessageSent
};
typedef unsigned SKPSMTPState;
    
// Message part keys
#define kSKPSMTPPartContentDispositionKey		@"kSKPSMTPPartContentDispositionKey"
#define kSKPSMTPPartContentTypeKey				@"kSKPSMTPPartContentTypeKey"
#define kSKPSMTPPartMessageKey					@"kSKPSMTPPartMessageKey"
#define kSKPSMTPPartContentTransferEncodingKey	@"kSKPSMTPPartContentTransferEncodingKey"

// Error message codes
#define kSKPSMPTErrorConnectionTimeout -5
#define kSKPSMTPErrorConnectionFailed -3
#define kSKPSMTPErrorConnectionInterrupted -4
#define kSKPSMTPErrorUnsupportedLogin -2
#define kSKPSMTPErrorTLSFail -1
#define kSKPSMTPErrorInvalidUserPass 535
#define kSKPSMTPErrorInvalidMessage 550
#define kSKPSMTPErrorInvalidMessage554 554
#define kSKPSMTPErrorNoRelay 530

#define BOUNDARY_STRING @"SimpleSMTP--05-13-2009"

@class SKPSMTPMessage;

@protocol SKPSMTPMessageDelegate
@required

- (void)messageSent:(SKPSMTPMessage *)message;
- (void)messageFailed:(SKPSMTPMessage *)message error:(NSError *)error;
- (void)verboseMessage:(NSString *)msg;

@end

@interface SKPSMTPMessage : NSObject <NSCopying>
{
    NSString *login;
    NSString *pass;
    NSString *relayHost;
    NSArray *relayPorts;
    
    NSString *subject;
    NSString *fromEmail;
    NSString *toEmail;
	NSString *ccEmail;
	NSString *bccEmail;
    NSArray *parts;
	NSString *aux;
    
    BOOL requiresAuth;
    BOOL wantsSecure;
    BOOL validateSSLChain;
    
    id <SKPSMTPMessageDelegate> delegate;
    
    NSTimeInterval connectTimeout;
    
    NSOutputStream *outputStream;
    NSInputStream *inputStream;
    
    SKPSMTPState sendState;
    BOOL isSecure;
    
    // Auth support flags
    BOOL serverAuthCRAMMD5;
    BOOL serverAuthPLAIN;
    BOOL serverAuthLOGIN;
    BOOL serverAuthDIGESTMD5;
    
    // Content support flags
    BOOL server8bitMessages;
    
    NSMutableString *inputString;
    NSTimer *connectTimer;
    NSTimer *watchdogTimer;
}

- (BOOL)send;

// Accessors
- (NSString *)login;
- (void)setLogin:(NSString *)value;

- (NSString *)pass;
- (void)setPass:(NSString *)value;

- (NSString *)relayHost;
- (void)setRelayHost:(NSString *)value;

- (NSArray *)relayPorts;
- (void)setRelayPorts:(NSArray *)value;

- (NSString *)subject;
- (void)setSubject:(NSString *)value;

- (NSString *)fromEmail;
- (void)setFromEmail:(NSString *)value;

- (NSString *)toEmail;
- (void)setToEmail:(NSString *)value;

- (NSString *)ccEmail;
- (void)setCcEmail:(NSString *)value;

- (NSString *)bccEmail;
- (void)setBccEmail:(NSString *)value;

- (NSArray *)parts;
- (void)setParts:(NSArray *)value;

- (NSString *)aux;
- (void)setAux:(NSString *)value;

- (BOOL)requiresAuth;
- (void)setRequiresAuth:(BOOL)value;

- (BOOL)wantsSecure;
- (void)setWantsSecure:(BOOL)value;

- (BOOL)validateSSLChain;
- (void)setValidateSSLChain:(BOOL)value;

- (NSTimeInterval)connectTimeout;
- (void)setConnectTimeout:(NSTimeInterval)value;

- (id <SKPSMTPMessageDelegate>)delegate;
- (void)setDelegate:(id <SKPSMTPMessageDelegate>)newDelegate;

@end
